/*
* gc global variable provides access to GUI Composer infrastructure components and project information.
* For more information, please see the Working with Javascript guide in the online help.
*/
var gc = gc || {};
gc.services = gc.services || {};
var saveAsFilePath; // undefined initially

// [ Start of file stream code
var mystream;
var mywriter;
var myTextEnc = new TextEncoder();
var saveStart = function(filename) {
	if (!filename) filename = 'mydata.csv';
	mystream = streamSaver.createWriteStream(filename);
	saveAbort();
	mywriter = mystream.getWriter();
	saveData(myTextEnc.encode('Temperature, Time\n'));
};
var saveStop = function() {
	if (mywriter) {
		mywriter.close();
		mywriter = null;
	}
};
var saveAbort = function() {
	if (mywriter){
		mywriter.abort('reason');
		mywriter = null;
	}
};
var saveData = function(data) {
	if (mywriter) {
		mywriter.write(data);
	}
};
window.addEventListener('unload', (event) => {
	//  console.log('unload', event);
	saveAbort(); // seems ok but please test.
});
var saveDummyData = function() {
	// To save textual data use TextEncoder to encode the data first
	var my_text = 'abcdef 01234 ABCDEF 56789 some text to go ... done';
	var myary = myTextEnc.encode(my_text + '\n');
	saveData(myary);
	// To save byte array data like below, no need to encode it.
	//saveData([1,2,3,4,5,6,7,8,9,10, 100,101]);  
};
// End of file stream code ]

var CF = 0

/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function() {
	/*
	*   Add custom computed value databindings here, using the following method:
	*
	*   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
	*
	*   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
	*   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
	*/

	// For example, a simple computed values based on simple expression
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

	// Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

	// Event 1 to n bindings
	/*
	gc.databind.registry.bind('widget.date.value',
		// dependant bindings needed in order to compute the date, in name/value pairs.
		{
			weekday: 'widget.dayOfWeek.selectedText',
			day: 'widget.dayOfMonth.value',
			month: 'widget.month.selectedText',
			year: 'widget.year.value'
		},
		// getter for date computation
		function(values)
		{
			// compute and return the string value to bind to the widget with id 'date'
			return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
		}
	);
	*/ 
});

/*
*  Boilerplate code for creating custom actions
*/
document.addEventListener('gc-nav-ready', function() {
	/*
	*   Add custom actions for menu items using the following api:
	*
	*   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
	*
	*   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
	*   param runable - function that performs the custom action.
	*   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
	*   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
	*/

	// For example,
	// gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

	// Alternatively, to programmatically disable a menu action at any time use:
	// gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction');

	gc.nav.registerAction('show_introduction', function() {
		if ((templateObj) && (templateObj.$)) {
			templateObj.$.ti_widget_vtabcontainer.selectedIndex = 0;
			templateObj.$.ti_widget_menuitem.hideFullMenu();
		}
	}, function() {
		return true;
	}, function() {
		return true;
	});
	gc.nav.registerAction('show_registers', function() {
		if ((templateObj) && (templateObj.$)) {
			templateObj.$.ti_widget_vtabcontainer.selectedIndex = 1;
			templateObj.$.ti_widget_menuitem.hideFullMenu();
		}
	}, function() {
		return true;
	}, function() {
		return true;
	});
	gc.nav.registerAction('show_motor_control', function() {
		if ((templateObj) && (templateObj.$)) {
			templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
			templateObj.$.ti_widget_menuitem.hideFullMenu();
		}
	}, function() {
		return true;
	}, function() {
		return true;
	});

	gc.nav.registerAction('load_registers', function() {
		if ((templateObj) && (templateObj.$)) {
			gc.File.browseAndLoad(null, {
bin: false
			}, function(contents, properties) {
				var regData = null;
				try {
					regData = JSON.parse(contents);
					if (regData) {
						templateObj.setRegisterData(regData);
						templateObj.$.ti_widget_eventlog_view.log("info", "Load Registers: loaded from :" + properties.localPath);
					}
				} catch (ex) {
					templateObj.logMsgAndNotify("error", "Load Registers: Could not parse file:" + ex, false, true);
				}

			});
		}
	}, function() {
		return true;
	}, function() {
		return true;
	});

	var saveText = function(fname, text, mime) {
		var blob = new Blob([text], {
type: mime
		});
		saveAs(blob, fname);
		return false;
	}

	var saveRegsAs = function(calledBy, regDataJson) {
		gc.File.browseAndSave(regDataJson, 'regs.json', '.json', function(result, errorInfo) {
			if (errorInfo) {
				templateObj.logMsgAndNotify("error", calledBy + " " + errorInfo, false, true);
			} else if ((result) && (result.localPath)) {
				saveAsFilePath = result.localPath;
				templateObj.$.ti_widget_eventlog_view.log("info", "Registers saved to " + result.localPath);
			}
		});
	}

	gc.nav.registerAction('save_registers', function() {
		if ((templateObj) && (templateObj.$)) {
			var regData = templateObj.getRegisterData();
			var regDataJson = JSON.stringify(regData);
			if (gc.desktop.isDesktop()) {
				if (!saveAsFilePath) {
					saveRegsAs("Save Registers", regDataJson);
				} else {
					gc.File.save(regDataJson, {"localPath": saveAsFilePath}, null, function(errorInfo) {
						if (errorInfo) {
							templateObj.logMsgAndNotify("error", "Save Registers As: " + errorInfo, false, true);
						} else {
							templateObj.$.ti_widget_eventlog_view.log("info", "Registers saved to " + saveAsFilePath);
						}
					});
				}

			} else {
				saveText("regs.json", regDataJson, 'text/plain;charset=utf-8')
				templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
			}
		}
	}, function() {
		return true;
	}, function() {
		return true;
	});

	gc.nav.registerAction('save_registers_as', function() {
		if ((templateObj) && (templateObj.$)) {
			var regData = templateObj.getRegisterData();
			var regDataJson = JSON.stringify(regData);
			if (gc.desktop.isDesktop()) {
				saveRegsAs("Save Registers As", regDataJson);
			} else {
				saveText("regs.json", regDataJson, 'text/plain;charset=utf-8')
				templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers As: regs.json file downloaded");
			}
		}
	}, function() {
		return true;
	}, function() {
		return true;
	});

	gc.nav.registerAction('open_log_pane', function() {
		if ((templateObj) && (templateObj.$)) {
			templateObj.$.ti_widget_eventlog_view.openView();
		}
	}, function() {
		return true;
	}, function() {
		return true;
	});

	gc.nav.registerAction('open_scripting_dialog', function() {
		if ((templateObj) && (templateObj.$)) {
			templateObj.$.scripting_dialog.removeAttribute('hidden');
			templateObj.$.scripting_dialog.open();
		}
	}, function() {
		return true;
	}, function() {
		return true;
	});

});

/*
*  Boilerplate code for working with components in the application gist
*/


var initComplete = false;
var templateObj;
var registerModel;


// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
	templateObj = document.querySelector('#template_obj');

	// Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
	// before trying to access components in the application.
	templateObj.addEventListener('dom-change', function() {
		if (initComplete) return;
		this.async(function() {
			initComplete = true;
			console.log("Application template has been stamped.");
			templateObj.$.ti_widget_toast.hideToast();
			templateObj.$.ti_widget_eventlog_view.log("info", "Application started.");
			registerModel = Polymer.dom(document).querySelector('ti-model-register');
			if (registerModel) {
				registerModel.addEventListener("usbHidModelReceivingData", function (event) {
					templateObj.logMsgAndNotify("debug", "Receiving data from target.", false, false);
				});
			} else {
				console.log("no ti-model-register component found in this application!")
			}

			// Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
			// e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

			// open the eventlog viewer when the user clicks on the book icon in the status bar:
			templateObj.$.ti_widget_eventlog_view.closeView(); // ensure that view states are in sync with application
			templateObj.$.ti_widget_statusbar.addEventListener("appiconclicked", function(event) {
				templateObj.$.ti_widget_eventlog_view.toggleView();
			});


			// Expand vtabcontainer nav bar when user clicks on menu icon or 'Menu' label
			templateObj.toggleMenu = function(event){
				templateObj.$.ti_widget_vtabcontainer.isExpanded = !templateObj.$.ti_widget_vtabcontainer.isExpanded;
			};
			templateObj.$.ti_widget_icon_button_menu.addEventListener('click',templateObj.toggleMenu);
			templateObj.$.ti_widget_label_menu.addEventListener('click',templateObj.toggleMenu);

			if (templateObj.$.ti_widget_register_page) {
				// get the active device name from the register page
				templateObj.$.ti_widget_menubar.productName = templateObj.$.ti_widget_register_page.deviceName;

				// called when the user clicks the Write Register button:
				templateObj.$.ti_widget_register_page.addEventListener("write_reg", function (event) {
					var detailsObj = event.detail;
					templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
				}.bind(this));
				
				// called when the user clicks the Write All Registers button:
				templateObj.$.ti_widget_register_page.addEventListener("write_regs", function (event) {
					var detailsObj = event.detail;
					templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
				}.bind(this));                

				// called when the user clicks the Read Register button:
				templateObj.$.ti_widget_register_page.addEventListener("read_reg", function (event) {
					var detailsObj = event.detail;
					templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
				}.bind(this));

				// called when the user clicks the Read All Registers button:
				templateObj.$.ti_widget_register_page.addEventListener("read_regs", function (event) {
					var detailsObj = event.detail;
					templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
				}.bind(this));

				// called when the user changes the immediate / deferred mode
				templateObj.$.ti_widget_register_page.addEventListener("update_mode_changed", function (event) {
					var detailsObj = event.detail;
					templateObj.$.ti_widget_eventlog_view.log("info", "request to change update mode to " + detailsObj.mode.label);
				}.bind(this));

				// called when the user changes the auto read refresh rate
				templateObj.$.ti_widget_register_page.addEventListener("auto_read_config_changed", function (event) {
					var detailsObj = event.detail;
					templateObj.$.ti_widget_eventlog_view.log("info", "request to change automatic read interval to " + detailsObj.autoRead.label);
				}.bind(this));
			}

			var usbHidService = gc.services['ti-service-usbhid'];
			if (usbHidService) {
				usbHidService.addEventListener('usbHidError', function(event) {
					var detailsObj = event.detail;
					templateObj.logMsgAndNotify("error", detailsObj.info + ": " + detailsObj.error, false, true);
				})
			}

			templateObj.getRegisterData = function() {
				var regData = {
signature: "oneui-register-data",
data: []
				}
				var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
				for (var i = 0; i < regBlocks.length; i++) {
					var registersData = [];
					for (var r = 0; r < regBlocks[i].registers.length; r++) {
						var reg = regBlocks[i].registers[r];
						var regName = reg.name.toLowerCase();
						regName = regName.replace(/\&/g, "_");
						var item = {
idx: r,
id: regName,
value: reg.value
						}
						registersData.push(item);
					}
					regData.data.push(registersData);
				}
				var strJson = JSON.stringify(regData);
				return strJson;
			};

			templateObj.setRegisterData = function(strJson) {
				var regData = JSON.parse(strJson);
				var ok = true;
				var regBlocks = templateObj.$.ti_widget_register_page._registerView.regBlocks;
				for (var b = 0; b < regBlocks.length; b++) {
					var registersData = [];
					for (var r = 0; r < regBlocks[b].registers.length; r++) {
						var reg = regBlocks[b].registers[r];
						var regName = reg.name.toLowerCase();
						regName = regName.replace(/\&/g, "_");
						item = regData.data[b][r];
						if (item.id === regName) {
							reg.value = item.value;
							if (!templateObj.$.ti_widget_register_page.isDeferredWrite) {
								reg.valueLastWritten = reg.value;
								var intValue = gc.utils.getValueFromHexString("" + reg.value);
								templateObj.$.ti_widget_register_page.writeValueToTargetRegister(intValue, null, b, r);
							} else {
								templateObj.$.ti_widget_register_page._registerView.refreshRegValue(b,r);
							}
						} else {
							var msg = "load registers failed: register name mismatch (" + item.id + " does not match " + regName + ")";
							templateObj.logMsgAndNotify("warning", msg, false, true);
						}
					}
				}
				if (templateObj.$.ti_widget_register_page.isDeferredWrite){
					templateObj.$.ti_widget_register_page._registerView._deferredAccessCtr++; // update the page
				}
			};

			templateObj.showToastMsg = function(msgType, msg, durationInSeconds) {
				templateObj.$.ti_widget_toast.hideToast();
				templateObj.$.ti_widget_toast.message = msg;
				switch (msgType.toLowerCase()) {
				case "error":
					templateObj.$.ti_widget_toast.iconName = "error";
					templateObj.$.ti_widget_toast.backgroundColor = "red";
					break;
				case "warning":
					templateObj.$.ti_widget_toast.iconName = "warning";
					templateObj.$.ti_widget_toast.backgroundColor = "orange";
					break;
				default:
					templateObj.$.ti_widget_toast.iconName = "check";
					templateObj.$.ti_widget_toast.backgroundColor = "teal";
					break;
				}
				templateObj.$.ti_widget_toast.duration = +durationInSeconds;
				templateObj.$.ti_widget_toast.showToast();
				console.log("toast msg [" + msgType + "]: " + msg);
			};

			templateObj.logMsgAndNotify = function(msgType, msg, showInStatusBar, showInToast) {
				var iconName = templateObj.$.ti_widget_eventlog_view.iconForItem(msgType);
				if (showInStatusBar) {
					var statusBar = gc.services['ti-widget-statusbar'];
					if (statusBar) {
						statusBar.statusString3 = msg;
						statusBar.setIcon(iconName);
					}
				}
				if (showInToast) {
					templateObj.showToastMsg(msgType, msg, 4);
				}
				templateObj.$.ti_widget_eventlog_view.log(msgType, msg);
			};
			templateObj.$.ti_widget_label_learnmore.addEventListener("click",function(event){
				templateObj.$.ti_widget_label_productname.fontColor="#444";
				templateObj.$.ti_widget_label_productdesc.fontColor="#444";
				templateObj.$.ti_widget_label_learnmore.setAttribute('hidden',true);
				templateObj.$.ti_widget_label_showless.removeAttribute('hidden');
				templateObj.$.ti_widget_vtab_home.style.backgroundColor = "white";
				templateObj.$.ti_widget_tabcontainer.selectedIndex = 1;
			});
			templateObj.$.ti_widget_label_showless.addEventListener("click",function(event){
				templateObj.$.ti_widget_label_productname.fontColor="white";
				templateObj.$.ti_widget_label_productdesc.fontColor="white";
				templateObj.$.ti_widget_label_learnmore.removeAttribute('hidden');
				templateObj.$.ti_widget_label_showless.setAttribute('hidden',true);
				templateObj.$.ti_widget_vtab_home.style.backgroundColor = "rgb(20, 140, 156)";
				templateObj.$.ti_widget_tabcontainer.selectedIndex = 0;
			});
			if (templateObj.$.ti_widget_button_connect) {
				templateObj.$.ti_widget_button_connect.addEventListener("click",function(event){
					if (gc.connectionManager){
						gc.connectionManager.connect();
					}
				});
			}
			
			templateObj.$.ti_widget_droplist2_i2c_freq.addEventListener("selected", function(event) {
				var u2a= document.querySelector('ti-model-register').getCodec();         // get codec
				var Cmd_I2C_Set = 1;                                                     // Command to set I2C port
				var speed = templateObj.$.ti_widget_droplist2_i2c_freq.selectedIndex;    // Speed based on droplist index: 0:100kHz, 1:400kHz, 2:10kHz
				var commandSent = u2a.sendCommandPacket(Cmd_I2C_Set, [speed, 0, 1])      // send command to set I2C, see packet definition in USB2ANY.h
				//don't implement 2:10kHz because it will cause GC hardware connection to time out
			});

			gc.databind.registry.bind('ti_widget_droplist_auto_read.selectedIndex', "ti_widget_register_page.autoReadValue", 
			function(value){                                // Getter function, sets drop list based on autoReadValue changes
				switch(Number(value)){
				case -1: //Auto read is off
					return 0;
				case 0: // As Fast as possible
					return 1;
				case 15: // 15ms delay
					return 2;
				case 50: // 50mS delay
					return 3;
				case 100: // 100ms delay
					return 4;
				case 500: // 500mS delay
					return 5;
				case 1000: // 1000mS delay
					return 6;
				default:
					return 4;
				}
			},
			function(value){                                // Setter function, sets autoReadValue based on droplist chnages
				switch(Number(value)){
				case 0: //Auto read is off
					return -1;
				case 1: // As Fast as possible
					return 0;
				case 2: // 15ms delay
					return 15;
				case 3: // 50mS delay
					return 50;
				case 4: // 100ms delay
					return 100;
				case 5: // 500mS delay
					return 500;
				case 6: // 1000mS delay
					return 1000;
				default:
					return 100;
				}
			});
			gc.databind.registry.bind('ti_widget_droplist_i2c_addr.selectedValue','reg.$deviceAddrs');
			gc.databind.registry.bind('widget.ti_widget_label_temp.label',"reg.TEMP_RESULT.TEMP + '°C'");
			gc.databind.registry.bind('widget.ti_widget_scalargraph.value_0','reg.TEMP_RESULT.TEMP', 
			function(value) {
				var d = new Date();
				timestamp = d.getFullYear() + '/' + d.getMonth() + '/' + d.getDate() + ' ' + d.getHours() + ':' + d.getMinutes() + ':' + d.getSeconds();
				saveData(myTextEnc.encode(value + ',' + timestamp + '\n'));
				return (value);
			}
			); 
			gc.databind.registry.bind('widget.ti_widget_textbox_high_limit_C.value','(reg.ALERT_HI_LIMIT.THIGH)',
			function(value) {
				return value + '°C';
			},
			function(value) {
				return parseFloat(value);
			}
			);
			gc.databind.registry.bind('widget.ti_widget_textbox_high_limit_hex.value','reg.ALERT_HI_LIMIT',
			function(value) { 
				return '0x' + value.toString(16).toUpperCase();
			},
			function(value) {
				return parseInt(value,16)
			}
			);
			gc.databind.registry.bind('widget.ti_widget_textbox_low_limit_C.value','(reg.ALERT_LO_LIMIT.TLOW)',
			function(value) { 
				return value + '°C';
			},
			function(value) {
				return parseFloat(value);
			}
			);
			gc.databind.registry.bind('widget.ti_widget_textbox_low_limit_hex.value','reg.ALERT_LO_LIMIT',
			function(value) { 
				return '0x' + value.toString(16).toUpperCase();
			},
			function(value) {
				return parseInt(value,16);
			}
			);
			//I2C General Call Reset
			templateObj.$.ti_widget_button_i2c_gcr.addEventListener("click", function(event) {
				var u2a = document.querySelector('ti-model-register').getCodec();
				var Cmd_I2C_Write = 2;
				var commandSent = u2a.sendCommandPacket(Cmd_I2C_Write, [0, 0, 1, 6])
			});
			//SMBus Alert Query (read from address 0x19)
			templateObj.$.ti_widget_button_smbus_alert_query.addEventListener("click", function(event) {
				var u2a= document.querySelector('ti-model-register').getCodec();
				var Cmd_I2C_Read = 3;
				var commandSent = u2a.sendCommandPacket(Cmd_I2C_Read, [0, 0x19, 1,0])
				u2a.readResponse(commandSent).then(function(responsePacket) {
					var payloadLength = responsePacket[2];
					var payload = responsePacket.slice(8);
					window.alert(payload);
				});
			});
			//ONE-Shot 
			templateObj.$.ti_widget_button_send_one_shot.addEventListener("click", function(event) {
				var registerModel = gc.databind.registry.getModel("reg");
				var bind = registerModel.getBinding("CONFIG_REG");
				var configRegValue = bind.getValue() | 0x8000;
				registerModel.writeValue("CONFIG_REG", configRegValue);
				//bind.updateValue(configRegValue); //optional update register view
			});
		}, 1);
	});
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
	init();
} else {
	document.addEventListener('DOMContentLoaded', init);
}